import javax.microedition.lcdui.*;

/** 
 * The game screen.
 */
public class GameScreen extends Canvas implements CommandListener
{
    private Command m_cmd_new_easy;
    private Command m_cmd_new_medium;
    private Command m_cmd_new_hard;
    private Command m_cmd_stats;
    private Command m_cmd_about;
    private Command m_cmd_quit;
    
    private Storage m_storage;
    
    private Board m_board = new Board();
    private int m_difficulty = 0;
    
    private int m_cursor_x = 0;
    private int m_cursor_y = 0;
    
    public GameScreen()
    {
        int pri = 1;
        
        m_cmd_stats = new Command( "Statistics", Command.ITEM, pri++ );
        m_cmd_new_easy = new Command( "New (easy)", Command.ITEM, pri++ );
        m_cmd_new_medium = new Command( "New (medium)", Command.ITEM, pri++ );
        m_cmd_new_hard = new Command( "New (hard)", Command.ITEM, pri++ );
        m_cmd_about = new Command( "About", Command.ITEM, pri++ );
        m_cmd_quit = new Command( "Quit", Command.EXIT, 1 );
        
        addCommand( m_cmd_new_easy );
        addCommand( m_cmd_new_medium );
        addCommand( m_cmd_new_hard );
        addCommand( m_cmd_stats );
        addCommand( m_cmd_about );
        addCommand( m_cmd_quit );
        setCommandListener( this );
        
        // Restore the situation
        m_storage = new Storage();
        if ( m_storage.current_board == null )
            newBoard( 0 );
        else
        {
            m_board = new Board( m_storage.current_board );
            m_difficulty = m_storage.current_difficulty;
        }
    }

    /**
     * Returns the current data storage.
     * 
     * @return Returns the current data storage.
     */
    public Storage getStorage()
    {
        return m_storage;
    }
    
    /**
     * Generates a new board.
     * 
     * @param difficulty The difficulty setting.
     */
    public void newBoard( int difficulty )
    {
        m_difficulty = difficulty;
        m_board = null;
        
        m_cursor_x = 0;
        m_cursor_y = 0;
        
        repaint();
        
        Thread runner = new BoardGenerator();
        runner.start();
    }
    
    /**
     * Java command listener.
     *  
     * @param c The issued command.
     * @param d The current displayable.
     */
    public void commandAction( Command c, Displayable d )
    {
        if ( c == m_cmd_stats )
        {
            Res.display.setCurrent( new StatsScreen( this, m_storage ) );
            return;
        }
        
        if ( c == m_cmd_about )
        {
            Res.display.setCurrent( new AboutScreen( this ) );
            return;
        }
        
        if ( c == m_cmd_quit )
        {
            if ( m_storage.current_board != null && m_board != null )
                m_storage.current_board = new Board( m_board );
            m_storage.save();
            
            Res.midlet.notifyDestroyed();
            return;
        }

        //////
        if ( m_board == null )
            return;
        
        if ( c == m_cmd_new_easy )
            newBoard( 0 );
        
        if ( c == m_cmd_new_medium )
            newBoard( 1 );
        
        if ( c == m_cmd_new_hard )
            newBoard( 2 );
        
        repaint();
    }
    
    /**
     * Key press event.
     */
    protected void keyPressed( int code )
    {
        if ( m_board == null )
            return;
        
        int num = -1;
        switch ( code )
        {
        case KEY_NUM0:	num = 0; break;
        case KEY_NUM1:	num = 1; break;
        case KEY_NUM2:	num = 2; break;
        case KEY_NUM3:	num = 3; break;
        case KEY_NUM4:	num = 4; break;
        case KEY_NUM5:	num = 5; break;
        case KEY_NUM6:	num = 6; break;
        case KEY_NUM7:	num = 7; break;
        case KEY_NUM8:	num = 8; break;
        case KEY_NUM9:	num = 9; break;
        }
        
        // Number pressed?
        if ( num >= 0 )
        {
            if ( m_board.types[ m_cursor_x ][ m_cursor_y ] == Board.TYPE_NORMAL )
            {
                m_board.numbers[ m_cursor_x ][ m_cursor_y ] = num;
                
                // Completed?
                if ( m_board.isComplete() )
                {
                    // Mark all numbers as "intial"
                    for ( int y = 0; y < 9; ++y )
                    {
                        for ( int x = 0; x < 9; ++x )
                        {
                            m_board.types[ x ][ y ] = Board.TYPE_INITIAL;
                        }
                    }
                    
                    // Save the score
                    m_storage.completed[ m_storage.current_difficulty ]++;
                    m_storage.current_board = null;
                    m_storage.save();
                }
            }
        }
        else
        { // Normal handling
            int dx = 0;
            int dy = 0;
            
            switch ( getGameAction( code ) )
            {
            case LEFT:		dx = -1; break;
            case RIGHT:		dx = 1; break;
            case UP:		dy = -1; break;
            case DOWN:		dy = 1; break;
            }
            
            m_cursor_x += dx;
            m_cursor_y += dy;
            
            if ( m_cursor_x < 0 ) m_cursor_x += 9;
            if ( m_cursor_x >= 9 ) m_cursor_x -= 9;
            if ( m_cursor_y < 0 ) m_cursor_y += 9;
            if ( m_cursor_y >= 9 ) m_cursor_y -= 9;
        }
        
        // Redraw
        repaint();
    }
    
    /**
     * The paint method.
     */
    protected void paint( Graphics g )
    {
        int w = getWidth();
        int h = getHeight();
        
        Font font = Font.getFont( Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL );
        int font_h = font.getHeight();
        g.setFont( font );
        
        // Background
        g.setColor( 0xffffffff );
        g.fillRect( 0, 0, w, h );
        
        // Generating a board?
        if ( m_board == null )
        {
            int y = 3;
            
            font = Font.getDefaultFont();
            font_h = font.getHeight();
            g.setFont( font );
            
            g.setColor( 0xff000000 );
            g.drawString( "Generating puzzle", 3, y, Graphics.LEFT | Graphics.TOP ); y += font_h;
            g.drawString( "Please wait..", 3, y, Graphics.LEFT | Graphics.TOP ); y += font_h;
            return;
        }
        
        // Layout
        int side_size = Math.min( w, h ) - 4;
        int block_size = side_size / 9;
        side_size = block_size * 9;
        
        // Draw the blocks
        int ul_x = (w - side_size) / 2;
        int ul_y = (h - side_size) / 2;
        
        int by = ul_y;
        for ( int idx_y = 0; idx_y < 9; ++idx_y, by += block_size )
        {
            int bx = ul_x;
            for ( int idx_x = 0; idx_x < 9; ++idx_x, bx += block_size )
            {
                // Selected?
                if ( idx_x == m_cursor_x && idx_y == m_cursor_y )
                {
                    g.setColor( 0xffa0f0c0 );
                    g.fillRect( bx, by, block_size, block_size );
                }
                
                // The number
                int number = m_board.numbers[ idx_x ][ idx_y ];
                int number_col = 0xff3040ff;
                if ( m_board.isValid( idx_x, idx_y ) == false )
                    number_col = 0xffbb0000;
                if ( m_board.types[ idx_x ][ idx_y ] == Board.TYPE_INITIAL )
                    number_col = 0xff000000;
                
                if ( number > 0 )
                {
                    char nch = (char) ('0' + number);
                    
                    int numx = bx + (block_size - font.charWidth( nch )) / 2;
                    int numy = by + (block_size - font.getHeight()) / 2;
                    
                    if ( block_size <= 12 )
                    {
                        numx = bx + 3;
                        numy = by + 2;
                    }
                    
                    g.setColor( number_col );
                    g.drawChar( nch, numx, numy, Graphics.LEFT | Graphics.TOP );
                }
            }
        }

        // Draw the frame
        int xy = 0;
        for ( int idx_y = 0; idx_y <= 9; ++idx_y, xy += block_size )
        {
            int x = ul_x + xy;
            int y = ul_y + xy;
            
            g.setColor( 0xff000000 );
            
            boolean is_thick = ( idx_y % 3 ) == 0;
            
            // Horizontal
            g.drawLine( ul_x, y, ul_x + side_size, y );
            if ( is_thick )
            {
                g.drawLine( ul_x, y - 1, ul_x + side_size, y - 1 );
                g.drawLine( ul_x, y + 1, ul_x + side_size, y + 1 );
            }
            
            // Vertical
            g.drawLine( x, ul_y, x, ul_y + side_size );
            if ( is_thick )
            {
                g.drawLine( x - 1, ul_y, x - 1, ul_y + side_size );
                g.drawLine( x + 1, ul_y, x + 1, ul_y + side_size );
            }
        }
    }
    
    /**
     * A thread that generates new boards.
     */
    private class BoardGenerator extends Thread
    {
        public void run()
        {
            int diff = m_difficulty;
            
            try
            {
                Board b = new Board();
                b.generate( diff );
                
                // Set the board
                m_board = b;
                
                // Store the game
                m_storage.current_board = new Board( m_board );
                m_storage.current_difficulty = diff;
                m_storage.save();
            }
            catch ( Throwable t )
            {
                System.out.println( "Error generating board:" );
                t.printStackTrace();
            }
            
            if ( m_board == null )
                m_board = new Board();
            repaint();
        }
    }
}
