import javax.microedition.rms.*;
import java.io.*;

/**
 * Persistent data storage. 
 */
public class Storage
{
    private static final String STORE = "FreeDoku";
    private static final int VERSION = 1;
    
    public Board current_board = null;
    public int current_difficulty = 0;
    
    public int[] completed = new int[ 3 ];
    
    public Storage()
    {
        try
        {
            RecordStore rs = RecordStore.openRecordStore( STORE, false );
            
            // Get the data
            if ( rs.getNumRecords() <= 0 )
                return;
            RecordEnumeration enum = rs.enumerateRecords( null, null, false );
            byte[] data = enum.nextRecord();
            enum.destroy();
            
            // Read the data
            ByteArrayInputStream bais = new ByteArrayInputStream( data );
            DataInputStream in = new DataInputStream( bais );
            
            if ( in.readInt() != VERSION )
                return;
            
            if ( in.readBoolean() )
            {
                current_board = new Board();
                for ( int y = 0; y < 9; ++y )
                {
                    for ( int x = 0; x < 9; ++x )
                    {
                        current_board.numbers[ x ][ y ] = (int) in.readByte();
                        current_board.types[ x ][ y ] = (int) in.readByte();
                    }
                }
            }
            
            current_difficulty = in.readInt();
            
            for ( int i = 0; i < completed.length; ++i )
                completed[ i ] = in.readInt();
            
            in.close();
        }
        catch ( Throwable t )
        {
            t.printStackTrace();
        }
    }
    
    /**
     * Saves the current storage
     */
    public void save()
    {
        for ( int tries = 0; tries < 3; ++tries )
        {
            if ( trySave() )
                break;
        }
    }
    
    /**
     * Tries to save the state.
     * 
     * @return True if successfull, false otherwise.
     */
    private boolean trySave()
    {
        try
        {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            DataOutputStream out = new DataOutputStream( baos );
            
            out.writeInt( VERSION );
            
            out.writeBoolean( current_board != null );
            if ( current_board != null )
            {
                for ( int y = 0; y < 9; ++y )
                {
                    for ( int x = 0; x < 9; ++x )
                    {
                        out.writeByte( current_board.numbers[ x ][ y ] );
                        out.writeByte( current_board.types[ x ][ y ] );
                    }
                }
            }
            
            out.writeInt( current_difficulty );
            
            for ( int i = 0; i < completed.length; ++i )
                out.writeInt( completed[ i ] );
            
            byte[] data = baos.toByteArray();
            out.close();

            // Set the record
            RecordStore rs = RecordStore.openRecordStore( STORE, true );

            if ( rs.getNumRecords() > 0 )
            {
                RecordEnumeration enum = rs.enumerateRecords( null, null, false );
                int id = enum.nextRecordId();
                enum.destroy();
                
                rs.setRecord( id, data, 0, data.length );
            }
            else
            {
                rs.addRecord( data, 0, data.length );
            }
        }
        catch ( Throwable t )
        {
            t.printStackTrace();
            return false;
        }
        
        return true;
    }
}
